@@ -44,6 +44,7 @@
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libnautilus-private/nautilus-file-utilities.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -51,10 +52,10 @@
  * These are in microseconds.
  */
 #define AUTOSCROLL_TIMEOUT_INTERVAL 100
-#define AUTOSCROLL_INITIAL_DELAY 750000
+#define AUTOSCROLL_INITIAL_DELAY 100000
 
 /* drag this close to the view edge to start auto scroll*/
-#define AUTO_SCROLL_MARGIN 20
+#define AUTO_SCROLL_MARGIN 30
 
 /* the smallest amount of auto scroll used when we just enter the autoscroll
  * margin
@@ -69,16 +70,11 @@
 void
 nautilus_drag_init (NautilusDragInfo     *drag_info,
 		    const GtkTargetEntry *drag_types,
-		    int                   drag_type_count, 
-		    GdkBitmap            *stipple)
+		    int                   drag_type_count)
 {
 	drag_info->target_list = gtk_target_list_new (drag_types,
 						   drag_type_count);
 
-	if (stipple != NULL) {
-		drag_info->stipple = g_object_ref (stipple);
-	}
-
 	drag_info->drop_occured = FALSE;
 	drag_info->need_to_destroy = FALSE;
 }
@@ -89,10 +85,6 @@ nautilus_drag_finalize (NautilusDragInfo *drag_info)
 	gtk_target_list_unref (drag_info->target_list);
 	nautilus_drag_destroy_selection_list (drag_info->selection_list);
 
-	if (drag_info->stipple != NULL) {
-		g_object_unref (drag_info->stipple);
-	}
-
 	g_free (drag_info);
 }
 
@@ -228,12 +220,19 @@ nautilus_drag_items_local (const char *target_uri_string, const GList *selection
 
 	target_uri = gnome_vfs_uri_new (target_uri_string);
 
-	/* get the parent URI of the first item in the selection */
-	item_uri = gnome_vfs_uri_new (((NautilusDragSelectionItem *)selection_list->data)->uri);
-	result = gnome_vfs_uri_is_parent (target_uri, item_uri, FALSE);
-	
-	gnome_vfs_uri_unref (item_uri);
-	gnome_vfs_uri_unref (target_uri);
+	if (target_uri != NULL) {
+		/* get the parent URI of the first item in the selection */
+		item_uri = gnome_vfs_uri_new (((NautilusDragSelectionItem *)selection_list->data)->uri);
+		
+		if (item_uri != NULL) {
+			result = gnome_vfs_uri_is_parent (target_uri, item_uri, FALSE);
+			
+			gnome_vfs_uri_unref (item_uri);
+		}
+		
+		gnome_vfs_uri_unref (target_uri);
+	}
+		
 	
 	return result;
 }
@@ -249,6 +248,39 @@ nautilus_drag_items_in_trash (const GList *selection_list)
 	return eel_uri_is_in_trash (((NautilusDragSelectionItem *)selection_list->data)->uri);
 }
 
+gboolean
+nautilus_drag_items_on_desktop (const GList *selection_list)
+{
+	char *uri;
+	GnomeVFSURI *vfs_uri, *desktop_vfs_uri;
+	char *desktop_uri;
+	gboolean result;
+	
+	/* check if the first item on the list is in trash.
+	 * FIXME:
+	 * we should really test each item but that would be slow for large selections
+	 * and currently dropped items can only be from the same container
+	 */
+	uri = ((NautilusDragSelectionItem *)selection_list->data)->uri;
+	if (eel_uri_is_desktop (uri)) {
+		return TRUE;
+	}
+
+	vfs_uri = gnome_vfs_uri_new (uri);
+	desktop_uri = nautilus_get_desktop_directory_uri ();
+	desktop_vfs_uri = gnome_vfs_uri_new (desktop_uri);
+	g_free (desktop_uri);
+	
+	result = gnome_vfs_uri_is_parent (desktop_vfs_uri, vfs_uri, FALSE);
+	
+	gnome_vfs_uri_unref (desktop_vfs_uri);
+	gnome_vfs_uri_unref (vfs_uri);
+	
+	return result;
+	
+}
+
+
 void
 nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 	const char *target_uri_string, const GList *items,
@@ -302,12 +334,19 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 
 		return;
 
-	} else if (eel_str_has_prefix (target_uri_string, NAUTILUS_COMMAND_SPECIFIER)
-			|| eel_str_has_prefix (target_uri_string, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
+	} else if (g_str_has_prefix (target_uri_string, NAUTILUS_COMMAND_SPECIFIER)
+			|| g_str_has_prefix (target_uri_string, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
 		if (actions & GDK_ACTION_MOVE) {
 			*action = GDK_ACTION_MOVE;
 		}
 		return;
+	} else if (eel_uri_is_desktop (target_uri_string)) {
+		if (actions & GDK_ACTION_MOVE) {
+			*action = GDK_ACTION_MOVE;
+		} else {
+			*action = context->suggested_action;
+		}
+		return;
 	} else {
 		target_uri = gnome_vfs_uri_new (target_uri_string);
 	}
@@ -321,8 +360,10 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 	dropped_uri = gnome_vfs_uri_new (((NautilusDragSelectionItem *)items->data)->uri);
 	same_fs = TRUE;
 
-	gnome_vfs_check_same_fs_uris (dropped_uri, target_uri, &same_fs);
-	gnome_vfs_uri_unref (dropped_uri);
+	if (dropped_uri != NULL) {
+		gnome_vfs_check_same_fs_uris (dropped_uri, target_uri, &same_fs);
+		gnome_vfs_uri_unref (dropped_uri);
+	}
 	gnome_vfs_uri_unref (target_uri);
 	
 	if (same_fs) {
@@ -560,7 +601,8 @@ append_drop_action_menu_item (GtkWidget          *menu,
 
 /* Pops up a menu of actions to perform on dropped files */
 GdkDragAction
-nautilus_drag_drop_action_ask (GdkDragAction actions)
+nautilus_drag_drop_action_ask (GtkWidget *widget,
+			       GdkDragAction actions)
 {
 	GtkWidget *menu;
 	GtkWidget *menu_item;
@@ -570,6 +612,7 @@ nautilus_drag_drop_action_ask (GdkDragAction actions)
 	 * allowed actions.
 	 */
 	menu = gtk_menu_new ();
+	gtk_menu_set_screen (GTK_MENU (menu), gtk_widget_get_screen (widget));
 	
 	append_drop_action_menu_item (menu, _("_Move here"),
 				      GDK_ACTION_MOVE,
@@ -622,6 +665,61 @@ nautilus_drag_drop_action_ask (GdkDragAction actions)
 	return damd.chosen;
 }
 
+GdkDragAction
+nautilus_drag_drop_background_ask (GtkWidget *widget, 
+				   GdkDragAction actions)
+{
+	GtkWidget *menu;
+	GtkWidget *menu_item;
+	DropActionMenuData damd;
+	
+	/* Create the menu and set the sensitivity of the items based on the
+	 * allowed actions.
+	 */
+	menu = gtk_menu_new ();
+	gtk_menu_set_screen (GTK_MENU (menu), gtk_widget_get_screen (widget));
+	
+	append_drop_action_menu_item (menu, _("Set as background for _all folders"),
+				      NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND,
+				      (actions & NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND) != 0,
+				      &damd);
+
+	append_drop_action_menu_item (menu, _("Set as background for _this folder"),
+				      NAUTILUS_DND_ACTION_SET_AS_BACKGROUND,
+				      (actions & NAUTILUS_DND_ACTION_SET_AS_BACKGROUND) != 0,
+				      &damd);
+
+	menu_item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	gtk_widget_show (menu_item);
+	
+	menu_item = gtk_menu_item_new_with_mnemonic (_("Cancel"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	gtk_widget_show (menu_item);
+	
+	damd.chosen = 0;
+	damd.loop = g_main_loop_new (NULL, FALSE);
+
+	g_signal_connect (menu, "deactivate",
+			  G_CALLBACK (menu_deactivate_callback),
+			  &damd);
+	
+	gtk_grab_add (menu);
+	
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+			NULL, NULL, 0, GDK_CURRENT_TIME);
+	
+	g_main_loop_run (damd.loop);
+
+	gtk_grab_remove (menu);
+	
+	g_main_loop_unref (damd.loop);
+	
+	gtk_object_sink (GTK_OBJECT (menu));
+	
+	return damd.chosen;	
+}
+
 gboolean
 nautilus_drag_autoscroll_in_scroll_region (GtkWidget *widget)
 {
@@ -716,14 +814,14 @@ nautilus_drag_autoscroll_start (NautilusDragInfo *drag_info,
 			drag_info->start_auto_scroll_in = eel_get_system_time() 
 				+ AUTOSCROLL_INITIAL_DELAY;
 			
-			drag_info->auto_scroll_timeout_id = gtk_timeout_add
+			drag_info->auto_scroll_timeout_id = g_timeout_add
 				(AUTOSCROLL_TIMEOUT_INTERVAL,
 				 callback,
 			 	 user_data);
 		}
 	} else {
 		if (drag_info->auto_scroll_timeout_id != 0) {
-			gtk_timeout_remove (drag_info->auto_scroll_timeout_id);
+			g_source_remove (drag_info->auto_scroll_timeout_id);
 			drag_info->auto_scroll_timeout_id = 0;
 		}
 	}
@@ -733,7 +831,7 @@ void
 nautilus_drag_autoscroll_stop (NautilusDragInfo *drag_info)
 {
 	if (drag_info->auto_scroll_timeout_id != 0) {
-		gtk_timeout_remove (drag_info->auto_scroll_timeout_id);
+		g_source_remove (drag_info->auto_scroll_timeout_id);
 		drag_info->auto_scroll_timeout_id = 0;
 	}
 }
@@ -742,36 +840,15 @@ gboolean
 nautilus_drag_selection_includes_special_link (GList *selection_list)
 {
 	GList *node;
-	char *uri, *local_path;
-	gboolean link_in_selection;
-	GnomeVFSFileInfo *info;
-
-	link_in_selection = FALSE;
+	char *uri;
 
 	for (node = selection_list; node != NULL; node = node->next) {
 		uri = ((NautilusDragSelectionItem *) node->data)->uri;
 
-		/* FIXME bugzilla.gnome.org 43020: This does sync. I/O and works only locally. */
-		local_path = gnome_vfs_get_local_path_from_uri (uri);
-
-		if (local_path) {
-			info = gnome_vfs_file_info_new ();
-			gnome_vfs_get_file_info
-				(local_path, info, 
-				 GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
-				 GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-			/* assume info is blank on failure */
-			link_in_selection = (nautilus_link_local_is_trash_link (local_path, info) ||
-					     nautilus_link_local_is_home_link (local_path, info) ||
-					     nautilus_link_local_is_volume_link (local_path, info));
-			gnome_vfs_file_info_unref (info);
-			g_free (local_path);
-		}
-		
-		if (link_in_selection) {
-			break;
+		if (eel_uri_is_desktop (uri)) {
+			return TRUE;
 		}
 	}
 	
-	return link_in_selection;
+	return FALSE;
 }
