@@ -84,16 +84,6 @@
  */
 #define ICON_CACHE_SWEEP_TIMEOUT        (10 * 1000)
 
-/* Embedded text font size and text line settings */
-
-/* FIXME; Hard coded font size */
-#define EMBEDDED_TEXT_FONT_SIZE 9
-#define EMBEDDED_TEXT_LINE_SPACING      1
-#define EMBEDDED_TEXT_EMPTY_LINE_HEIGHT 4
-
-#define MINIMUM_EMBEDDED_TEXT_RECT_WIDTH	20
-#define MINIMUM_EMBEDDED_TEXT_RECT_HEIGHT	20
-
 /* This circular doubly-linked list structure is used to keep a list
  * of the most recently used items in the cache.
  */
@@ -107,7 +97,6 @@ struct CircularList {
 typedef struct {
 	char *name; /* Icon name or absolute filename */
 	char *modifier;
-	char *embedded_text;
 	guint nominal_size;
 } CacheKey;
 
@@ -117,6 +106,7 @@ typedef struct {
 	
 	GdkPixbuf *pixbuf;
 	GnomeIconData *icon_data;
+	time_t mtime; /* Only used for absolute filenames */
 
 	CircularList recently_used_node;
 } CacheIcon;
@@ -147,6 +137,8 @@ typedef struct {
         guint sweep_timer;
 
 	CacheIcon *fallback_icon;
+	GHashTable *image_mime_types;
+
 } NautilusIconFactory;
 
 #define NAUTILUS_ICON_FACTORY(obj) \
@@ -164,6 +156,7 @@ static guint signals[LAST_SIGNAL];
 
 
 static int cached_thumbnail_limit;
+static int show_image_thumbs;
 
 /* forward declarations */
 
@@ -172,6 +165,7 @@ static void       nautilus_icon_factory_class_init       (NautilusIconFactoryCla
 static void       nautilus_icon_factory_instance_init    (NautilusIconFactory      *factory);
 static void       nautilus_icon_factory_finalize         (GObject                  *object);
 static void       thumbnail_limit_changed_callback       (gpointer                  user_data);
+static void       show_thumbnails_changed_callback       (gpointer                  user_data);
 static void       mime_type_data_changed_callback        (GnomeVFSMIMEMonitor	   *monitor,
 							  gpointer                  user_data);
 static guint      cache_key_hash                         (gconstpointer             p);
@@ -183,11 +177,8 @@ static CacheIcon *cache_icon_new                         (GdkPixbuf
 							  GnomeIconData            *icon_data);
 static CacheIcon *get_icon_from_cache                    (const char               *icon,
 							  const char               *modifier,
-							  const char               *embedded_text,
 							  guint                     nominal_size);
-static GdkPixbuf *embed_text                             (GdkPixbuf                *pixbuf_without_text,
-							  GnomeIconData            *icon_data,
-							  const char               *text);
+static void nautilus_icon_factory_clear                  (void);
 
 GNOME_CLASS_BOILERPLATE (NautilusIconFactory,
 			 nautilus_icon_factory,
@@ -201,6 +192,9 @@ destroy_icon_factory (void)
 	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_IMAGE_FILE_THUMBNAIL_LIMIT,
 					 thumbnail_limit_changed_callback,
 					 NULL);
+	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
+					 show_thumbnails_changed_callback,
+					 NULL);
 	g_object_unref (global_icon_factory);
 }
 
@@ -219,6 +213,11 @@ get_icon_factory (void)
 					      thumbnail_limit_changed_callback,
 					      NULL);
 
+		show_thumbnails_changed_callback (NULL);
+		eel_preferences_add_callback (NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
+					      show_thumbnails_changed_callback,
+					      NULL);
+
 		g_signal_connect (gnome_vfs_mime_monitor_get (),
 				  "data_changed",
 				  G_CALLBACK (mime_type_data_changed_callback),
@@ -241,6 +240,8 @@ icon_theme_changed_callback (GnomeIconTheme *icon_theme,
 {
 	NautilusIconFactory *factory;
 
+	nautilus_icon_factory_clear ();
+
 	factory = user_data;
 
 	g_signal_emit (factory,
@@ -320,6 +321,14 @@ static void
 nautilus_icon_factory_instance_init (NautilusIconFactory *factory)
 {
 	GdkPixbuf *pixbuf;
+	guint i;
+	static const char *types [] = {
+		"image/x-bmp", "image/x-ico", "image/jpeg", "image/gif",
+		"image/png", "image/pnm", "image/ras", "image/tga",
+		"image/tiff", "image/wbmp", "image/x-xbitmap",
+		"image/x-xpixmap"
+        };
+
 	
         factory->icon_cache = g_hash_table_new_full (cache_key_hash,
 						     cache_key_equal,
@@ -352,6 +361,13 @@ nautilus_icon_factory_instance_init (NautilusIconFactory *factory)
 					   NULL);
 	
 	factory->fallback_icon = cache_icon_new (pixbuf, NULL);
+
+	factory->image_mime_types = g_hash_table_new (g_str_hash, g_str_equal);
+	for (i = 0; i < G_N_ELEMENTS (types); i++) {
+		g_hash_table_insert (factory->image_mime_types,
+				     (gpointer) types [i],
+				     GUINT_TO_POINTER (1));
+	}
 }
 
 static void
@@ -377,7 +393,6 @@ static void
 cache_key_destroy (CacheKey *key)
 {
 	g_free (key->name);
-	g_free (key->embedded_text);
 	g_free (key);
 }
 
@@ -395,7 +410,8 @@ cache_icon_new (GdkPixbuf     *pixbuf,
 	icon->ref_count = 1;
 	icon->pixbuf = pixbuf;
 	icon->icon_data = icon_data;
-
+	icon->mtime = 0;
+	
 	return icon;
 }
 
@@ -591,9 +607,19 @@ nautilus_icon_factory_clear (void)
 	
 	/* Empty out the recently-used list. */
 	head = &factory->recently_used_dummy_head;
-	g_assert (factory->recently_used_count == 0);
-	g_assert (head->next == head);
-	g_assert (head->prev == head);
+
+	/* fallback_icon hangs around, but we don't know if it
+	 * was ever inserted in the list
+	 */
+	g_assert (factory->recently_used_count == 0 ||
+		  factory->recently_used_count == 1);
+
+	if (factory->recently_used_count == 1) {
+		/* make sure this one is the fallback_icon */
+		g_assert (head->next == head->prev);
+		g_assert (&factory->fallback_icon->recently_used_node == head->next);
+	}
+
 }
 
 static void
@@ -617,6 +643,11 @@ nautilus_icon_factory_finalize (GObject *object)
 		g_assert (factory->fallback_icon->ref_count == 1);
 		cache_icon_unref (factory->fallback_icon);
 	}
+
+	if (factory->image_mime_types) {
+		g_hash_table_destroy (factory->image_mime_types);
+		factory->image_mime_types = NULL;
+	}
 	
 	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
@@ -630,10 +661,25 @@ thumbnail_limit_changed_callback (gpointer user_data)
 	 * signal to mean only "thumbnails might have changed" if this ends up being slow
 	 * for some reason.
 	 */
+	nautilus_icon_factory_clear ();
 	g_signal_emit (global_icon_factory,
 			 signals[ICONS_CHANGED], 0);
 }
 
+static void
+show_thumbnails_changed_callback (gpointer user_data)
+{
+	show_image_thumbs = eel_preferences_get_enum (NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS);
+
+	nautilus_icon_factory_clear ();
+	/* If the user disabled thumbnailing, remove all outstanding thumbnails */ 
+	if (show_image_thumbs == NAUTILUS_SPEED_TRADEOFF_NEVER) {
+		nautilus_thumbnail_remove_all_from_queue ();
+	}
+	g_signal_emit (global_icon_factory,
+		       signals[ICONS_CHANGED], 0);
+}
+
 static void       
 mime_type_data_changed_callback (GnomeVFSMIMEMonitor *monitor, gpointer user_data)
 {
@@ -643,6 +689,7 @@ mime_type_data_changed_callback (GnomeVFSMIMEMonitor *monitor, gpointer user_dat
 	/* We don't know which data changed, so we have to assume that
 	 * any or all icons might have changed.
 	 */
+	nautilus_icon_factory_clear ();
 	g_signal_emit (get_icon_factory (), 
 			 signals[ICONS_CHANGED], 0);
 }				 
@@ -652,7 +699,7 @@ nautilus_remove_icon_file_name_suffix (const char *icon_name)
 {
 	guint i;
 	const char *suffix;
-	static const char *icon_file_name_suffixes[] = { ".svg", ".png", ".jpg" };
+	static const char *icon_file_name_suffixes[] = { ".svg", ".svgz", ".png", ".jpg" };
 
 	for (i = 0; i < G_N_ELEMENTS (icon_file_name_suffixes); i++) {
 		suffix = icon_file_name_suffixes[i];
@@ -680,15 +727,52 @@ image_uri_to_name_or_uri (const char *image_uri)
 	return NULL;
 }
 
+static gboolean
+mimetype_limited_by_size (const char *mime_type)
+{
+	NautilusIconFactory *factory;
+
+	factory = get_icon_factory();
+
+        if (g_hash_table_lookup (factory->image_mime_types, mime_type)) {
+                return TRUE;
+	}
+
+        return FALSE;
+}
+
+static gboolean
+should_show_thumbnail (NautilusFile *file, const char *mime_type)
+{
+	if (mimetype_limited_by_size (mime_type) &&
+	    nautilus_file_get_size (file) > (unsigned int)cached_thumbnail_limit) {
+		return FALSE;
+	}
+	
+	if (show_image_thumbs == NAUTILUS_SPEED_TRADEOFF_ALWAYS) {
+		return TRUE;
+	} else if (show_image_thumbs == NAUTILUS_SPEED_TRADEOFF_NEVER) {
+		return FALSE;
+	} else {
+		/* only local files */
+		return nautilus_file_is_local (file);
+	}
+
+	return FALSE;
+}
+
 /* key routine to get the icon for a file */
 char *
-nautilus_icon_factory_get_icon_for_file (NautilusFile *file)
+nautilus_icon_factory_get_icon_for_file (NautilusFile *file, gboolean embedd_text)
 {
  	char *custom_uri, *file_uri, *icon_name, *mime_type, *custom_icon;
 	NautilusIconFactory *factory;
 	GnomeIconLookupResultFlags lookup_result;
 	GnomeVFSFileInfo *file_info;
-
+	GnomeThumbnailFactory *thumb_factory;
+	gboolean show_thumb;
+	GnomeIconLookupFlags lookup_flags;
+	
 	if (file == NULL) {
 		return NULL;
 	}
@@ -716,21 +800,33 @@ nautilus_icon_factory_get_icon_for_file (NautilusFile *file)
 	mime_type = nautilus_file_get_mime_type (file);
 	
 	file_info = nautilus_file_peek_vfs_file_info (file);
-		
+	
+	show_thumb = should_show_thumbnail (file, mime_type);	
+	
+	if (show_thumb) {
+		thumb_factory = factory->thumbnail_factory;
+	} else {
+		thumb_factory = NULL;
+	}
+
+	lookup_flags = GNOME_ICON_LOOKUP_FLAGS_SHOW_SMALL_IMAGES_AS_THEMSELVES;
+	if (embedd_text) {
+		lookup_flags |= GNOME_ICON_LOOKUP_FLAGS_EMBEDDING_TEXT;
+	}
 	icon_name = gnome_icon_lookup (factory->icon_theme,
-				       factory->thumbnail_factory,
+				       thumb_factory,
 				       file_uri,
 				       custom_icon,
 				       nautilus_file_peek_vfs_file_info (file),
 				       mime_type,
-				       GNOME_ICON_LOOKUP_FLAGS_EMBEDDING_TEXT |
-				       GNOME_ICON_LOOKUP_FLAGS_SHOW_SMALL_IMAGES_AS_THEMSELVES,
+				       lookup_flags,
 				       &lookup_result);
 
 
 	/* Create thumbnails if we can, and if the looked up icon isn't a thumbnail
 	   or an absolute pathname (custom icon or image as itself) */
-	if (!(lookup_result & GNOME_ICON_LOOKUP_RESULT_FLAGS_THUMBNAIL) &&
+	if (show_thumb &&
+	    !(lookup_result & GNOME_ICON_LOOKUP_RESULT_FLAGS_THUMBNAIL) &&
 	    icon_name[0] != '/' && file_info &&
 	    gnome_thumbnail_factory_can_thumbnail (factory->thumbnail_factory,
 						   file_uri,
@@ -748,41 +844,16 @@ nautilus_icon_factory_get_icon_for_file (NautilusFile *file)
 	return icon_name;
 }
 
-/**
- * nautilus_icon_factory_get_basic_file_attributes
- * 
- * Get the list of file attributes required to obtain a file's basic icon.
- * This includes attributes needed to get a custom icon, but not those needed
- * for text preview.
- * Callers must free this list.
- */
-GList *
-nautilus_icon_factory_get_basic_file_attributes (void)
-{
-	GList *attributes;
-
-	attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_CUSTOM_ICON);
-	attributes = g_list_prepend (attributes,
-				     NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE);
-	return attributes;
-}
-
 /**
  * nautilus_icon_factory_get_required_file_attributes
  * 
- * Get the list of file attributes required to obtain a file's icon.
- * Callers must free this list.
+ * Get the file attributes required to obtain a file's icon.
  */
-GList *
+NautilusFileAttributes 
 nautilus_icon_factory_get_required_file_attributes (void)
 {
-	GList *attributes;
-
-	attributes = nautilus_icon_factory_get_basic_file_attributes ();
-	attributes = g_list_prepend (attributes,
-				     NAUTILUS_FILE_ATTRIBUTE_TOP_LEFT_TEXT);
-
-	return attributes;
+	return NAUTILUS_FILE_ATTRIBUTE_CUSTOM_ICON |
+		NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE;
 }
 
 
@@ -797,34 +868,11 @@ nautilus_icon_factory_get_required_file_attributes (void)
 gboolean
 nautilus_icon_factory_is_icon_ready_for_file (NautilusFile *file)
 {
-	GList *attributes;
+	NautilusFileAttributes attributes;
 	gboolean result;
 
 	attributes = nautilus_icon_factory_get_required_file_attributes ();
 	result = nautilus_file_check_if_ready (file, attributes);
-	g_list_free (attributes);
-
-	return result;
-}
-
-/**
- * nautilus_icon_factory_is_basic_icon_ready_for_file
- * 
- * Check whether a NautilusFile has enough information to report
- * what its basic icon should be. This will account for custom icons
- * but not text preview.
- * 
- * @file: The NautilusFile in question.
- */
-gboolean
-nautilus_icon_factory_is_basic_icon_ready_for_file (NautilusFile *file)
-{
-	GList *attributes;
-	gboolean result;
-
-	attributes = nautilus_icon_factory_get_basic_file_attributes ();
-	result = nautilus_file_check_if_ready (file, attributes);
-	g_list_free (attributes);
 
 	return result;
 }
@@ -978,7 +1026,7 @@ load_pixbuf_svg (const char *path,
 	if (pixbuf == NULL) {
 		return NULL;
 	}
-	
+
 	if (icon_data != NULL) {
 		width = gdk_pixbuf_get_width (pixbuf);
 		height = gdk_pixbuf_get_height (pixbuf);
@@ -995,7 +1043,7 @@ path_represents_svg_image (const char *path)
 	 * and if they're doing that, they can behave themselves
 	 * and use a .svg extension.
 	 */
-	return path != NULL && strstr (path, ".svg") != NULL;
+	return path != NULL && (strstr (path, ".svg") != NULL || strstr (path, ".svgz") != NULL);
 }
 
 static GdkPixbuf *
@@ -1080,31 +1128,6 @@ load_icon_file (char          *filename,
 	return pixbuf;
 }
 
-static CacheIcon *
-create_embedded_text_cache_icon (CacheIcon  *base_icon,
-				 const char *embedded_text)
-{
-	GdkPixbuf *pixbuf;
-	GnomeIconData *icon_data;
-
-	pixbuf = embed_text (base_icon->pixbuf,
-			     base_icon->icon_data,
-			     embedded_text);
-
-	if (pixbuf) {
-		icon_data = NULL;
-		if (base_icon->icon_data) {
-			icon_data = g_new (GnomeIconData, 1);
-			*icon_data = *base_icon->icon_data;
-		}
-		return cache_icon_new (pixbuf, icon_data);
-	} else {
-		cache_icon_ref (base_icon);
-		return base_icon;
-	}
-}
-
-
 static CacheIcon *
 create_normal_cache_icon (const char *icon,
 			  const char *modifier,
@@ -1118,18 +1141,24 @@ create_normal_cache_icon (const char *icon,
 	CacheIcon *cache_icon;
 	GdkPixbuf *pixbuf;
 	int base_size;
+	struct stat statbuf;
+	time_t mtime;
 		
 	factory = get_icon_factory ();
 
 	icon_data = NULL;
 	filename = NULL;
 
+	mtime = 0;
+	
 	base_size = 0;
 	if (icon[0] == '/') {
 		/* FIXME: maybe we should add modifier to the filename
 		 *        before the extension */
-		if (g_file_test (icon, G_FILE_TEST_IS_REGULAR)) {
+		if (stat (icon, &statbuf) == 0 &&
+		    S_ISREG (statbuf.st_mode)) {
 			filename = g_strdup (icon);
+			mtime = statbuf.st_mtime;
 		}
 	} else {
 		if (modifier) {
@@ -1162,13 +1191,14 @@ create_normal_cache_icon (const char *icon,
 				 base_size,
 				 nominal_size,
 				 icon_data);
-	
+	g_free (filename);
 	if (pixbuf == NULL) {
 		return NULL;
 	}
 	
 	cache_icon = cache_icon_new (pixbuf, icon_data);
-
+	cache_icon->mtime = mtime;
+	
 	return cache_icon;
 }
 
@@ -1181,15 +1211,15 @@ create_normal_cache_icon (const char *icon,
 static CacheIcon *
 get_icon_from_cache (const char *icon,
 		     const char *modifier,
-		     const char *embedded_text,
 		     guint       nominal_size)
 {
 	NautilusIconFactory *factory;
 	GHashTable *hash_table;
 	CacheKey lookup_key;
 	CacheKey *key;
-	CacheIcon *cached_icon, *base_cached_icon;
+	CacheIcon *cached_icon;
 	gpointer key_in_table, value;
+	struct stat statbuf;
 	
 	g_return_val_if_fail (icon != NULL, NULL);
 
@@ -1202,7 +1232,6 @@ get_icon_from_cache (const char *icon,
 	/* Check to see if it's already in the table. */
 	lookup_key.name = (char *)icon;
 	lookup_key.modifier = (char *)modifier;
-	lookup_key.embedded_text = (char *)embedded_text;
 	lookup_key.nominal_size = nominal_size;
 
 	if (g_hash_table_lookup_extended (hash_table, &lookup_key,
@@ -1214,6 +1243,16 @@ get_icon_from_cache (const char *icon,
 		cached_icon = value;
 	}
 
+	/* Make sure that thumbnails and image-as-itself icons gets
+	   reloaded when they change: */
+	if (cached_icon && icon[0] == '/') {
+		if (stat (icon, &statbuf) != 0 ||
+		    !S_ISREG (statbuf.st_mode) ||
+		    statbuf.st_mtime != cached_icon->mtime) {
+			cached_icon = NULL;
+		}
+	}
+
 	if (cached_icon == NULL) {
 		/* Not in the table, so load the image. */
 		
@@ -1222,39 +1261,25 @@ get_icon_from_cache (const char *icon,
 			 icon, modifier?modifier:"", embedded_text?"<tl>":"", nominal_size);
 		*/
 		
-		if (embedded_text) {
-			base_cached_icon = get_icon_from_cache (icon,
-								modifier,
+		cached_icon = create_normal_cache_icon (icon,
+							modifier,
+							nominal_size);
+		/* Try to fallback without modifier */
+		if (cached_icon == NULL && modifier != NULL) {
+			cached_icon = create_normal_cache_icon (icon,
 								NULL,
 								nominal_size);
-			cached_icon = create_embedded_text_cache_icon (base_cached_icon,
-								       embedded_text);
-			cache_icon_unref (base_cached_icon);
-			
-		} else {
-			cached_icon = create_normal_cache_icon (icon,
-								modifier,
-								nominal_size);
-			/* Try to fallback without modifier */
-			if (cached_icon == NULL && modifier != NULL) {
-				cached_icon = create_normal_cache_icon (icon,
-									NULL,
-									nominal_size);
-			}
-		
-			if (cached_icon == NULL) {
-				cached_icon = factory->fallback_icon;
-				cache_icon_ref (cached_icon);
-			}
 		}
 		
-	
+		if (cached_icon == NULL) {
+			cached_icon = factory->fallback_icon;
+			cache_icon_ref (cached_icon);
+		}
 		
 		/* Create the key and icon for the hash table. */
 		key = g_new (CacheKey, 1);
 		key->name = g_strdup (icon);
 		key->modifier = g_strdup (modifier);
-		key->embedded_text = g_strdup (embedded_text);
 		key->nominal_size = nominal_size;
 
 		g_hash_table_insert (hash_table, key, cached_icon);
@@ -1275,9 +1300,9 @@ get_icon_from_cache (const char *icon,
 GdkPixbuf *
 nautilus_icon_factory_get_pixbuf_for_icon (const char                  *icon,
 					   const char                  *modifier,
-					   const char                  *embedded_text,
 					   guint                        nominal_size,
 					   NautilusEmblemAttachPoints  *attach_points,
+					   GdkRectangle                *embedded_text_rect,
 					   gboolean                     wants_default,
 					   char                       **display_name)
 {
@@ -1290,7 +1315,6 @@ nautilus_icon_factory_get_pixbuf_for_icon (const char                  *icon,
 	factory = get_icon_factory ();
 	cached_icon = get_icon_from_cache (icon,
 					   modifier,
-					   embedded_text,
 					   nominal_size);
 
 	if (attach_points != NULL) {
@@ -1306,6 +1330,20 @@ nautilus_icon_factory_get_pixbuf_for_icon (const char                  *icon,
 			attach_points->num_points = 0;
 		}
 	}
+	if (embedded_text_rect) {
+		if (cached_icon->icon_data != NULL &&
+		    cached_icon->icon_data->has_embedded_rect) {
+			embedded_text_rect->x = cached_icon->icon_data->x0;
+			embedded_text_rect->y = cached_icon->icon_data->y0;
+			embedded_text_rect->width = cached_icon->icon_data->x1 - cached_icon->icon_data->x0;
+			embedded_text_rect->height = cached_icon->icon_data->y1 - cached_icon->icon_data->y0;
+		} else {
+			embedded_text_rect->x = 0;
+			embedded_text_rect->y = 0;
+			embedded_text_rect->width = 0;
+			embedded_text_rect->height = 0;
+		}
+	}
 
 	if (display_name != NULL) {
 		if (cached_icon->icon_data != NULL &&
@@ -1344,10 +1382,6 @@ cache_key_hash (gconstpointer p)
 		hash ^= g_str_hash (key->modifier);
 	}
 		
-	if (key->embedded_text) {
-		hash ^= g_str_hash (key->embedded_text);
-	}
-	
 	return hash;
 }
 
@@ -1361,8 +1395,7 @@ cache_key_equal (gconstpointer a, gconstpointer b)
 
 	return eel_strcmp (key_a->name, key_b->name) == 0 &&
 		key_a->nominal_size ==  key_b->nominal_size &&
-		eel_strcmp (key_a->modifier, key_b->modifier) == 0 &&
-		eel_strcmp (key_a->embedded_text, key_b->embedded_text) == 0;
+		eel_strcmp (key_a->modifier, key_b->modifier) == 0;
 }
 
 /* Return nominal icon size for given zoom level.
@@ -1390,7 +1423,7 @@ nautilus_get_icon_size_for_zoom_level (NautilusZoomLevel zoom_level)
 	case NAUTILUS_ZOOM_LEVEL_LARGEST:
 		return NAUTILUS_ICON_SIZE_LARGEST;
 	}
-	g_return_val_if_fail (FALSE, NAUTILUS_ICON_SIZE_STANDARD);
+	g_return_val_if_reached (NAUTILUS_ICON_SIZE_STANDARD);
 }
 
 /* Convenience cover for nautilus_icon_factory_get_icon_for_file
@@ -1403,20 +1436,18 @@ nautilus_icon_factory_get_pixbuf_for_file (NautilusFile *file,
 {
 	char *icon;
 	GdkPixbuf *pixbuf;
-	char *embedded_text;
 
 
 	/* Get the pixbuf for this file. */
-	icon = nautilus_icon_factory_get_icon_for_file (file);
+	icon = nautilus_icon_factory_get_icon_for_file (file, FALSE);
 	if (icon == NULL) {
 		return NULL;
 	}
-	embedded_text = nautilus_file_peek_top_left_text (file);
 
 	pixbuf = nautilus_icon_factory_get_pixbuf_for_icon (icon, modifier,
-							    embedded_text,
 							    size_in_pixels,
-							    NULL, TRUE, NULL);
+							    NULL, NULL,
+							    TRUE, NULL);
 	
 	g_free (icon);
 
@@ -1431,9 +1462,9 @@ nautilus_icon_factory_get_pixbuf_from_name (const char *icon_name,
 					    char **display_name)
 {
 	return nautilus_icon_factory_get_pixbuf_for_icon (icon_name, modifier,
-							  NULL,
 							  size_in_pixels,
-							  NULL, TRUE, display_name);
+							  NULL, NULL,
+							  TRUE, display_name);
 }
 									  
 GdkPixbuf *
@@ -1445,7 +1476,6 @@ nautilus_icon_factory_get_thumbnail_frame (void)
 gboolean
 nautilus_icon_factory_remove_from_cache (const char *icon_name,
 					 const char *modifier,
-					 const char *embedded_text,
 					 guint size)
 {
 	GHashTable *hash_table;
@@ -1458,118 +1488,11 @@ nautilus_icon_factory_remove_from_cache (const char *icon_name,
 	/* Check to see if it's already in the table. */
 	lookup_key.name = (char *)icon_name;
 	lookup_key.modifier = (char *)modifier;
-	lookup_key.embedded_text = (char *)embedded_text;
 	lookup_key.nominal_size = size;
 	
 	return g_hash_table_remove (hash_table, &lookup_key);
 }
 					 
-
-static gboolean
-embedded_text_rect_usable (GnomeIconData *icon_data)
-{
-	if (icon_data == NULL || !icon_data->has_embedded_rect) {
-		return FALSE;
-	}
-
-	if (icon_data->x1 - icon_data->x0 < MINIMUM_EMBEDDED_TEXT_RECT_WIDTH ||
-	    icon_data->y1 - icon_data->y0 < MINIMUM_EMBEDDED_TEXT_RECT_HEIGHT) {
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-static gboolean embedded_text_preferences_callbacks_added = FALSE;
-static PangoFontDescription *embedded_text_font = NULL;
-
-static void
-embedded_text_font_changed_callback (GtkSettings *settings,
-				     GParamSpec  *pspec,
-				     gpointer     callback_data)
-{
-	gboolean clear_cache;
-	char *font_name;
-	
-	clear_cache = GPOINTER_TO_INT (callback_data);
-
-	if (embedded_text_font != NULL) {
-		pango_font_description_free (embedded_text_font);
-	}
-
-	font_name = NULL;
-	g_object_get (settings,
-		      "gtk-font-name", &font_name,
-		      NULL);
-
-	if (font_name == NULL) {
-		font_name = g_strdup ("Sans");
-	}
-	
-	embedded_text_font = pango_font_description_from_string (font_name);
-	g_free (font_name);
-
-	pango_font_description_set_size (embedded_text_font,
-					 EMBEDDED_TEXT_FONT_SIZE * PANGO_SCALE);
-
-	if (clear_cache) {
-		nautilus_icon_factory_clear ();
-	}
-}
-
-static GdkPixbuf *
-embed_text (GdkPixbuf *pixbuf_without_text,
-	    GnomeIconData *icon_data,
-	    const char *text)
-{
-	GdkPixbuf *pixbuf_with_text;
-	PangoLayout *layout;
-	static PangoContext *context;
-	GtkSettings *settings;
-	ArtIRect clip_rect;
-	
-	g_return_val_if_fail (pixbuf_without_text != NULL, NULL);
-	
-	/* Quick out for the case where there's no place to embed the
-	 * text or the place is too small or there's no text.
-	 */
-	if (!embedded_text_rect_usable (icon_data) || eel_strlen (text) == 0) {
-		return NULL;
-	}
-
-	/* Listen for changes in embedded text (icon text preview) font preferences */
-	if (!embedded_text_preferences_callbacks_added) {
-		embedded_text_preferences_callbacks_added = TRUE;
-
-		settings = gtk_settings_get_default ();	
-		g_signal_connect (settings, "notify::gtk-font-name",
-				  G_CALLBACK (embedded_text_font_changed_callback), GINT_TO_POINTER (TRUE));
-		embedded_text_font_changed_callback (settings, NULL, GINT_TO_POINTER (FALSE));
-	}
-	
-	if (context == NULL) {
-		context = eel_pango_ft2_get_context ();
-		eel_debug_call_at_shutdown_with_data (g_object_unref, context);
-	}
-	
-	layout = pango_layout_new (context);
-	pango_layout_set_font_description (layout, embedded_text_font);
-	pango_layout_set_text (layout, text, -1);
-	
-	pixbuf_with_text = gdk_pixbuf_copy (pixbuf_without_text);
-	clip_rect.x0 = icon_data->x0;
-	clip_rect.y0 = icon_data->y0;
-	clip_rect.x1 = icon_data->x1;
-	clip_rect.y1 = icon_data->y1;
-	
-	eel_gdk_pixbuf_draw_layout_clipped (pixbuf_with_text,
-					    clip_rect,
-					    EEL_RGB_COLOR_BLACK, layout);
-	g_object_unref (layout);
-
-	return pixbuf_with_text;
-}
-
 #if ! defined (NAUTILUS_OMIT_SELF_CHECK)
 
 void
